@@ -1,24 +1,11 @@
 /*
- * This file is part of KDE.
- *
- * Copyright (c) 2001,2002,2003 Cornelius Schumacher <schumacher@kde.org>
- * Copyright (c) 2003 Waldo Bastian <bastian@kde.org>
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
- *
- * You should have received a copy of the GNU Library General Public License
- * along with this library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
- */
+    This file is part of KDE.
+
+    SPDX-FileCopyrightText: 2001, 2002, 2003 Cornelius Schumacher <schumacher@kde.org>
+    SPDX-FileCopyrightText: 2003 Waldo Bastian <bastian@kde.org>
+
+    SPDX-License-Identifier: LGPL-2.0-or-later
+*/
 
 #ifndef KCORECONFIGSKELETON_H
 #define KCORECONFIGSKELETON_H
@@ -34,6 +21,7 @@
 #include <QStringList>
 #include <QVariant>
 #include <QUrl>
+
 class KCoreConfigSkeletonPrivate;
 
 class KConfigSkeletonItemPrivate;
@@ -52,6 +40,7 @@ class KConfigSkeletonItemPrivate;
  */
 class KCONFIGCORE_EXPORT KConfigSkeletonItem
 {
+    Q_DECLARE_PRIVATE(KConfigSkeletonItem)
 public:
     typedef QList < KConfigSkeletonItem * >List;
     typedef QHash < QString, KConfigSkeletonItem * > Dict;
@@ -80,6 +69,22 @@ public:
      */
     QString group() const;
 
+    /**
+     * Set config file group but giving the KConfigGroup.
+     * Allow the item to be in nested groups.
+     * @since 5.68
+     */
+    void setGroup(const KConfigGroup &cg);
+
+    /**
+     * Return a KConfigGroup, the one provided by setGroup(KConfigGroup) if it's valid,
+     * or make one from @param config and item's group
+     * @sa setGroup(const QString &_group)
+     * @sa setGroup(KConfigGroup cg)
+     * @since 5.68
+     */
+    KConfigGroup configGroup(KConfig *config) const;
+
     /**
      * Set config file key.
      */
@@ -132,6 +137,19 @@ public:
     */
     QString whatsThis() const;
 
+    /**
+      The write flags to be used when writing configuration.
+      @since 5.58
+    */
+    void setWriteFlags(KConfigBase::WriteConfigFlags flags);
+
+    /**
+      Return write flags to be used when writing configuration.
+      They should be passed to every call of writeEntry() and revertToDefault().
+      @since 5.58
+    */
+    KConfigBase::WriteConfigFlags writeFlags() const;
+
     /**
      * This function is called by @ref KCoreConfigSkeleton to read the value for this setting
      * from a config file.
@@ -141,6 +159,7 @@ public:
     /**
      * This function is called by @ref KCoreConfigSkeleton to write the value of this setting
      * to a config file.
+     * Make sure to pass writeFlags() to every call of writeEntry() and revertToDefault().
      */
     virtual void writeConfig(KConfig *) = 0;
 
@@ -196,7 +215,30 @@ public:
      */
     bool isImmutable() const;
 
+    /**
+     * Indicates if the item is set to its default value.
+     *
+     * @since 5.64
+     */
+    bool isDefault() const;
+
+    /**
+     * Indicates if the item has a different value than the
+     * previously loaded value.
+     *
+     * @since 5.64
+     */
+    bool isSaveNeeded() const;
+
+    /**
+     * Returns the default value
+     * @since 5.74
+     */
+    QVariant getDefault() const;
+
 protected:
+    explicit KConfigSkeletonItem(KConfigSkeletonItemPrivate &dd, const QString &_group, const QString &_key);
+
     /**
      * sets mIsImmutable to true if mKey in config is immutable
      * @param group KConfigGroup to check if mKey is immutable in
@@ -207,10 +249,71 @@ protected:
     QString mKey; ///< The config key for this item
     QString mName; ///< The name of this item
 
-private:
-    KConfigSkeletonItemPrivate *const d;
+    // HACK: Necessary to avoid introducing new virtuals in KConfigSkeletonItem
+    // KF6: Use proper pure virtuals in KConfigSkeletonItem
+    void setIsDefaultImpl(const std::function<bool()> &impl);
+    void setIsSaveNeededImpl(const std::function<bool()> &impl);
+    void setGetDefaultImpl(const std::function<QVariant()> &impl);
+
+    KConfigSkeletonItemPrivate *const d_ptr;
 };
 
+class KPropertySkeletonItemPrivate;
+
+/**
+ * \class KPropertySkeletonItem kcoreconfigskeleton.h <KCoreConfigSkeleton>
+ *
+ * @short Class for proxying a QObject property as a preferences setting
+ * @author Kevin Ottens
+ * @see KConfigSkeletonItem
+ *
+ * This class represents one preferences setting as used by @ref KCoreConfigSkeleton.
+ * Unlike other @ref KConfigSkeletonItem subclasses, this one won't store the preference
+ * in KConfig but will use a QObject property as storage.
+ * You will have to register instances of this class with the function KCoreConfigSkeleton::addItem().
+ *
+ * @since 5.65
+ */
+class KCONFIGCORE_EXPORT KPropertySkeletonItem : public KConfigSkeletonItem
+{
+    Q_DECLARE_PRIVATE(KPropertySkeletonItem)
+public:
+    /**
+     * Constructor
+     *
+     * @param object The QObject instance which we'll manage the property of
+     * @param propertyName The name of the property in @p object which we'll manage
+     * @param defaultValue The default value of the property
+     */
+    KPropertySkeletonItem(QObject *object, const QByteArray &propertyName, const QVariant &defaultValue);
+
+    /** @copydoc KConfigSkeletonItem::property() */
+    QVariant property() const override;
+    /** @copydoc KConfigSkeletonItem::setProperty(const QVariant &) */
+    void setProperty(const QVariant &p) override;
+    /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) */
+    bool isEqual(const QVariant &p) const override;
+
+    /** @copydoc KConfigSkeletonItem::readConfig(KConfig *) */
+    void readConfig(KConfig *) override;
+    /** @copydoc KConfigSkeletonItem::writeConfig(KConfig *) */
+    void writeConfig(KConfig *) override;
+
+    /** @copydoc KConfigSkeletonItem::readDefault(KConfig *) */
+    void readDefault(KConfig *) override;
+    /** @copydoc KConfigSkeletonItem::setDefault() */
+    void setDefault() override;
+    /** @copydoc KConfigSkeletonItem::swapDefault() */
+    void swapDefault() override;
+
+    /**
+     * Set a notify function, it will be invoked when the value of the property changes.
+     * @since 5.68
+     */
+    void setNotifyFunction(const std::function<void ()> &impl);
+};
+
+
 /**
  * \class KConfigSkeletonGenericItem kcoreconfigskeleton.h <KConfigSkeletonGenericItem>
  */
@@ -226,6 +329,10 @@ public:
         : KConfigSkeletonItem(_group, _key), mReference(reference),
           mDefault(defaultValue), mLoadedValue(defaultValue)
     {
+        setIsDefaultImpl([this] { return mReference == mDefault; });
+        setIsSaveNeededImpl([this] { return mReference != mLoadedValue; });
+        setGetDefaultImpl([this] { return QVariant::fromValue(mDefault); });
+
     }
 
     /**
@@ -263,27 +370,27 @@ public:
     /**
       Set the value for this item to the default value
      */
-    void setDefault() Q_DECL_OVERRIDE
+    void setDefault() override
     {
         mReference = mDefault;
     }
 
     /** @copydoc KConfigSkeletonItem::writeConfig(KConfig *) */
-    void writeConfig(KConfig *config) Q_DECL_OVERRIDE
+    void writeConfig(KConfig *config) override
     {
         if (mReference != mLoadedValue) { // Is this needed?
-            KConfigGroup cg(config, mGroup);
+            KConfigGroup cg = configGroup(config);
             if ((mDefault == mReference) && !cg.hasDefault(mKey)) {
-                cg.revertToDefault(mKey);
+                cg.revertToDefault(mKey, writeFlags());
             } else {
-                cg.writeEntry(mKey, mReference);
+                cg.writeEntry(mKey, mReference, writeFlags());
             }
             mLoadedValue = mReference;
         }
     }
 
     /** @copydoc KConfigSkeletonItem::readDefault(KConfig*) */
-    void readDefault(KConfig *config) Q_DECL_OVERRIDE
+    void readDefault(KConfig *config) override
     {
         config->setReadDefaults(true);
         readConfig(config);
@@ -292,7 +399,7 @@ public:
     }
 
     /** @copydoc KConfigSkeletonItem::swapDefault() */
-    void swapDefault() Q_DECL_OVERRIDE
+    void swapDefault() override
     {
         T tmp = mReference;
         mReference = mDefault;
@@ -332,16 +439,27 @@ public:
     */
     KConfigCompilerSignallingItem(KConfigSkeletonItem *item, QObject* object,
             NotifyFunction targetFunction, quint64 userData);
-    virtual ~KConfigCompilerSignallingItem();
+    ~KConfigCompilerSignallingItem() override;
+
+    void readConfig(KConfig *) override;
+    void writeConfig(KConfig *) override;
+    void readDefault(KConfig *) override;
+    void setProperty(const QVariant &p) override;
+    bool isEqual(const QVariant &p) const override;
+    QVariant property() const override;
+    void setDefault() override;
+    void swapDefault() override;
+    // KF6 TODO - fix this
+    // Ideally we would do this in an overload of KConfigSkeletonItem, but
+    // given we can't, I've shadowed the method. This isn't pretty, but given
+    // the docs say it should generally only be used from auto generated code,
+    // should be fine.
+    void setWriteFlags(KConfigBase::WriteConfigFlags flags);
+    KConfigBase::WriteConfigFlags writeFlags() const;
+    void setGroup(const KConfigGroup &cg);
+    KConfigGroup configGroup(KConfig *config) const;
+    // END TODO
 
-    void readConfig(KConfig *) Q_DECL_OVERRIDE;
-    void writeConfig(KConfig *) Q_DECL_OVERRIDE;
-    void readDefault(KConfig *) Q_DECL_OVERRIDE;
-    void setProperty(const QVariant &p) Q_DECL_OVERRIDE;
-    bool isEqual(const QVariant &p) const Q_DECL_OVERRIDE;
-    QVariant property() const Q_DECL_OVERRIDE;
-    void setDefault() Q_DECL_OVERRIDE;
-    void swapDefault() Q_DECL_OVERRIDE;
 private:
     inline void invokeNotifyFunction()
     {
@@ -449,19 +567,19 @@ public:
                    Type type = Normal);
 
         /** @copydoc KConfigSkeletonItem::writeConfig(KConfig*) */
-        void writeConfig(KConfig *config) Q_DECL_OVERRIDE;
+        void writeConfig(KConfig *config) override;
 
         /** @copydoc KConfigSkeletonItem::readConfig(KConfig*) */
-        void readConfig(KConfig *config) Q_DECL_OVERRIDE;
+        void readConfig(KConfig *config) override;
 
         /** @copydoc KConfigSkeletonItem::setProperty(const QVariant&) */
-        void setProperty(const QVariant &p) Q_DECL_OVERRIDE;
+        void setProperty(const QVariant &p) override;
 
         /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) const */
-        bool isEqual(const QVariant &p) const Q_DECL_OVERRIDE;
+        bool isEqual(const QVariant &p) const override;
 
         /** @copydoc KConfigSkeletonItem::property() const */
-        QVariant property() const Q_DECL_OVERRIDE;
+        QVariant property() const override;
 
     private:
         Type mType;
@@ -505,19 +623,19 @@ public:
                 const QUrl &defaultValue = QUrl());
 
         /** @copydoc KConfigSkeletonItem::writeConfig(KConfig*) */
-        void writeConfig(KConfig *config) Q_DECL_OVERRIDE;
+        void writeConfig(KConfig *config) override;
 
         /** @copydoc KConfigSkeletonItem::readConfig(KConfig*) */
-        void readConfig(KConfig *config) Q_DECL_OVERRIDE;
+        void readConfig(KConfig *config) override;
 
         /** @copydoc KConfigSkeletonItem::setProperty(const QVariant&) */
-        void setProperty(const QVariant &p) Q_DECL_OVERRIDE;
+        void setProperty(const QVariant &p) override;
 
         /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) const */
-        bool isEqual(const QVariant &p) const Q_DECL_OVERRIDE;
+        bool isEqual(const QVariant &p) const override;
 
         /** @copydoc KConfigSkeletonItem::property() const */
-        QVariant property() const Q_DECL_OVERRIDE;
+        QVariant property() const override;
     };
 
     /**
@@ -530,14 +648,14 @@ public:
         ItemProperty(const QString &_group, const QString &_key,
                      QVariant &reference, const QVariant &defaultValue = QVariant());
 
-        void readConfig(KConfig *config) Q_DECL_OVERRIDE;
-        void setProperty(const QVariant &p) Q_DECL_OVERRIDE;
+        void readConfig(KConfig *config) override;
+        void setProperty(const QVariant &p) override;
 
         /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) const */
-        bool isEqual(const QVariant &p) const Q_DECL_OVERRIDE;
+        bool isEqual(const QVariant &p) const override;
 
         /** @copydoc KConfigSkeletonItem::property() const */
-        QVariant property() const Q_DECL_OVERRIDE;
+        QVariant property() const override;
     };
 
     /**
@@ -551,16 +669,16 @@ public:
                  bool defaultValue = true);
 
         /** @copydoc KConfigSkeletonItem::readConfig(KConfig*) */
-        void readConfig(KConfig *config) Q_DECL_OVERRIDE;
+        void readConfig(KConfig *config) override;
 
         /** @copydoc KConfigSkeletonItem::setProperty(const QVariant&) */
-        void setProperty(const QVariant &p) Q_DECL_OVERRIDE;
+        void setProperty(const QVariant &p) override;
 
         /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) const */
-        bool isEqual(const QVariant &p) const Q_DECL_OVERRIDE;
+        bool isEqual(const QVariant &p) const override;
 
         /** @copydoc KConfigSkeletonItem::property() const */
-        QVariant property() const Q_DECL_OVERRIDE;
+        QVariant property() const override;
     };
 
     /**
@@ -574,22 +692,22 @@ public:
                 qint32 defaultValue = 0);
 
         /** @copydoc KConfigSkeletonItem::readConfig(KConfig*) */
-        void readConfig(KConfig *config) Q_DECL_OVERRIDE;
+        void readConfig(KConfig *config) override;
 
         /** @copydoc KConfigSkeletonItem::setProperty(const QVariant&) */
-        void setProperty(const QVariant &p) Q_DECL_OVERRIDE;
+        void setProperty(const QVariant &p) override;
 
         /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) const */
-        bool isEqual(const QVariant &p) const Q_DECL_OVERRIDE;
+        bool isEqual(const QVariant &p) const override;
 
         /** @copydoc KConfigSkeletonItem::property() */
-        QVariant property() const Q_DECL_OVERRIDE;
+        QVariant property() const override;
 
         /** Get the minimum value that is allowed to be stored in this item */
-        QVariant minValue() const Q_DECL_OVERRIDE;
+        QVariant minValue() const override;
 
         /** Get the maximum value this is allowed to be stored in this item */
-        QVariant maxValue() const Q_DECL_OVERRIDE;
+        QVariant maxValue() const override;
 
         /** Set the minimum value for the item
             @sa minValue()
@@ -619,22 +737,22 @@ public:
                      qint64 defaultValue = 0);
 
         /** @copydoc KConfigSkeletonItem::readConfig(KConfig*) */
-        void readConfig(KConfig *config) Q_DECL_OVERRIDE;
+        void readConfig(KConfig *config) override;
 
         /** @copydoc KConfigSkeletonItem::setProperty(const QVariant&) */
-        void setProperty(const QVariant &p) Q_DECL_OVERRIDE;
+        void setProperty(const QVariant &p) override;
 
         /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) const */
-        bool isEqual(const QVariant &p) const Q_DECL_OVERRIDE;
+        bool isEqual(const QVariant &p) const override;
 
         /** @copydoc KConfigSkeletonItem::property() */
-        QVariant property() const Q_DECL_OVERRIDE;
+        QVariant property() const override;
 
         /** @copydoc ItemInt::minValue() */
-        QVariant minValue() const Q_DECL_OVERRIDE;
+        QVariant minValue() const override;
 
         /** @copydoc ItemInt::maxValue() */
-        QVariant maxValue() const Q_DECL_OVERRIDE;
+        QVariant maxValue() const override;
 
         /** @copydoc ItemInt::setMinValue(qint32) */
         void setMinValue(qint64);
@@ -648,8 +766,8 @@ public:
         qint64 mMin;
         qint64 mMax;
     };
-#ifndef KDE_NO_DEPRECATED
-    typedef KCONFIGCORE_DEPRECATED ItemLongLong ItemInt64;
+#if KCONFIGCORE_ENABLE_DEPRECATED_SINCE(5, 0)
+    typedef KCONFIGCORE_DEPRECATED_VERSION(5, 0, "Use ItemLongLong") ItemLongLong ItemInt64;
 #endif
 
     /**
@@ -674,15 +792,26 @@ public:
         QList<Choice> choices() const;
 
         /** @copydoc KConfigSkeletonItem::readConfig(KConfig*) */
-        void readConfig(KConfig *config) Q_DECL_OVERRIDE;
+        void readConfig(KConfig *config) override;
 
         /** @copydoc KConfigSkeletonItem::writeConfig(KConfig*) */
-        void writeConfig(KConfig *config) Q_DECL_OVERRIDE;
+        void writeConfig(KConfig *config) override;
 
         // Source compatibility with 4.x
+        // TODO KF6 remove
         typedef Choice Choice2;
         QList<Choice> choices2() const;
 
+        /**
+         * Returns the value for for the choice with the given name
+         */
+        QString valueForChoice(const QString &name) const;
+
+        /**
+         * Stores a choice value for name
+         */
+        void setValueForChoice(const QString &name, const QString &valueForChoice);
+
     private:
         QList<Choice> mChoices;
     };
@@ -698,22 +827,22 @@ public:
                  quint32 &reference, quint32 defaultValue = 0);
 
         /** @copydoc KConfigSkeletonItem::readConfig(KConfig*) */
-        void readConfig(KConfig *config) Q_DECL_OVERRIDE;
+        void readConfig(KConfig *config) override;
 
         /** @copydoc KConfigSkeletonItem::setProperty(const QVariant&) */
-        void setProperty(const QVariant &p) Q_DECL_OVERRIDE;
+        void setProperty(const QVariant &p) override;
 
         /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) const */
-        bool isEqual(const QVariant &p) const Q_DECL_OVERRIDE;
+        bool isEqual(const QVariant &p) const override;
 
         /** @copydoc KConfigSkeletonItem::property() */
-        QVariant property() const Q_DECL_OVERRIDE;
+        QVariant property() const override;
 
         /** @copydoc ItemInt::minValue() */
-        QVariant minValue() const Q_DECL_OVERRIDE;
+        QVariant minValue() const override;
 
         /** @copydoc ItemInt::maxValue() */
-        QVariant maxValue() const Q_DECL_OVERRIDE;
+        QVariant maxValue() const override;
 
         /** @copydoc ItemInt::setMinValue(qint32) */
         void setMinValue(quint32);
@@ -739,22 +868,22 @@ public:
                       quint64 defaultValue = 0);
 
         /** @copydoc KConfigSkeletonItem::readConfig(KConfig*) */
-        void readConfig(KConfig *config) Q_DECL_OVERRIDE;
+        void readConfig(KConfig *config) override;
 
         /** @copydoc KConfigSkeletonItem::setProperty(const QVariant&) */
-        void setProperty(const QVariant &p) Q_DECL_OVERRIDE;
+        void setProperty(const QVariant &p) override;
 
         /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) const */
-        bool isEqual(const QVariant &p) const Q_DECL_OVERRIDE;
+        bool isEqual(const QVariant &p) const override;
 
         /** @copydoc KConfigSkeletonItem::property() */
-        QVariant property() const Q_DECL_OVERRIDE;
+        QVariant property() const override;
 
         /** @copydoc ItemInt::minValue() */
-        QVariant minValue() const Q_DECL_OVERRIDE;
+        QVariant minValue() const override;
 
         /** @copydoc ItemInt::maxValue() */
-        QVariant maxValue() const Q_DECL_OVERRIDE;
+        QVariant maxValue() const override;
 
         /** @copydoc ItemInt::setMinValue(qint32) */
         void setMinValue(quint64);
@@ -768,8 +897,8 @@ public:
         quint64 mMin;
         quint64 mMax;
     };
-#ifndef KDE_NO_DEPRECATED
-    typedef KCONFIGCORE_DEPRECATED ItemULongLong ItemUInt64;
+#if KCONFIGCORE_ENABLE_DEPRECATED_SINCE(5, 0)
+    typedef KCONFIGCORE_DEPRECATED_VERSION(5, 0, "Use ItemULongLong") ItemULongLong ItemUInt64;
 #endif
 
     /**
@@ -783,22 +912,22 @@ public:
                    double &reference, double defaultValue = 0);
 
         /** @copydoc KConfigSkeletonItem::readConfig(KConfig*) */
-        void readConfig(KConfig *config) Q_DECL_OVERRIDE;
+        void readConfig(KConfig *config) override;
 
         /** @copydoc KConfigSkeletonItem::setProperty(const QVariant&) */
-        void setProperty(const QVariant &p) Q_DECL_OVERRIDE;
+        void setProperty(const QVariant &p) override;
 
         /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) const */
-        bool isEqual(const QVariant &p) const Q_DECL_OVERRIDE;
+        bool isEqual(const QVariant &p) const override;
 
         /** @copydoc KConfigSkeletonItem::property() */
-        QVariant property() const Q_DECL_OVERRIDE;
+        QVariant property() const override;
 
         /** @copydoc ItemInt::minValue() */
-        QVariant minValue() const Q_DECL_OVERRIDE;
+        QVariant minValue() const override;
 
         /** @copydoc ItemInt::maxValue() */
-        QVariant maxValue() const Q_DECL_OVERRIDE;
+        QVariant maxValue() const override;
 
         /** @copydoc ItemInt::setMinValue() */
         void setMinValue(double);
@@ -824,16 +953,16 @@ public:
                  const QRect &defaultValue = QRect());
 
         /** @copydoc KConfigSkeletonItem::readConfig(KConfig*) */
-        void readConfig(KConfig *config) Q_DECL_OVERRIDE;
+        void readConfig(KConfig *config) override;
 
         /** @copydoc KConfigSkeletonItem::setProperty(const QVariant&) */
-        void setProperty(const QVariant &p) Q_DECL_OVERRIDE;
+        void setProperty(const QVariant &p) override;
 
         /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) const */
-        bool isEqual(const QVariant &p) const Q_DECL_OVERRIDE;
+        bool isEqual(const QVariant &p) const override;
 
         /** @copydoc KConfigSkeletonItem::property() */
-        QVariant property() const Q_DECL_OVERRIDE;
+        QVariant property() const override;
     };
 
     /**
@@ -847,16 +976,16 @@ public:
                   const QPoint &defaultValue = QPoint());
 
         /** @copydoc KConfigSkeletonItem::readConfig(KConfig*) */
-        void readConfig(KConfig *config) Q_DECL_OVERRIDE;
+        void readConfig(KConfig *config) override;
 
         /** @copydoc KConfigSkeletonItem::setProperty(const QVariant&) */
-        void setProperty(const QVariant &p) Q_DECL_OVERRIDE;
+        void setProperty(const QVariant &p) override;
 
         /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) const */
-        bool isEqual(const QVariant &p) const Q_DECL_OVERRIDE;
+        bool isEqual(const QVariant &p) const override;
 
         /** @copydoc KConfigSkeletonItem::property() */
-        QVariant property() const Q_DECL_OVERRIDE;
+        QVariant property() const override;
     };
 
     /**
@@ -870,16 +999,16 @@ public:
                  const QSize &defaultValue = QSize());
 
         /** @copydoc KConfigSkeletonItem::readConfig(KConfig*) */
-        void readConfig(KConfig *config) Q_DECL_OVERRIDE;
+        void readConfig(KConfig *config) override;
 
         /** @copydoc KConfigSkeletonItem::setProperty(const QVariant&) */
-        void setProperty(const QVariant &p) Q_DECL_OVERRIDE;
+        void setProperty(const QVariant &p) override;
 
         /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) const */
-        bool isEqual(const QVariant &p) const Q_DECL_OVERRIDE;
+        bool isEqual(const QVariant &p) const override;
 
         /** @copydoc KConfigSkeletonItem::property() */
-        QVariant property() const Q_DECL_OVERRIDE;
+        QVariant property() const override;
     };
 
     /**
@@ -894,16 +1023,16 @@ public:
                      const QDateTime &defaultValue = QDateTime());
 
         /** @copydoc KConfigSkeletonItem::readConfig(KConfig*) */
-        void readConfig(KConfig *config) Q_DECL_OVERRIDE;
+        void readConfig(KConfig *config) override;
 
         /** @copydoc KConfigSkeletonItem::setProperty(const QVariant&) */
-        void setProperty(const QVariant &p) Q_DECL_OVERRIDE;
+        void setProperty(const QVariant &p) override;
 
         /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) const */
-        bool isEqual(const QVariant &p) const Q_DECL_OVERRIDE;
+        bool isEqual(const QVariant &p) const override;
 
         /** @copydoc KConfigSkeletonItem::property() */
-        QVariant property() const Q_DECL_OVERRIDE;
+        QVariant property() const override;
     };
 
     /**
@@ -918,16 +1047,16 @@ public:
                        const QStringList &defaultValue = QStringList());
 
         /** @copydoc KConfigSkeletonItem::readConfig(KConfig*) */
-        void readConfig(KConfig *config) Q_DECL_OVERRIDE;
+        void readConfig(KConfig *config) override;
 
         /** @copydoc KConfigSkeletonItem::setProperty(const QVariant&) */
-        void setProperty(const QVariant &p) Q_DECL_OVERRIDE;
+        void setProperty(const QVariant &p) override;
 
         /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) const */
-        bool isEqual(const QVariant &p) const Q_DECL_OVERRIDE;
+        bool isEqual(const QVariant &p) const override;
 
         /** @copydoc KConfigSkeletonItem::property() */
-        QVariant property() const Q_DECL_OVERRIDE;
+        QVariant property() const override;
     };
 
     /**
@@ -942,9 +1071,9 @@ public:
                      const QStringList &defaultValue = QStringList());
 
         /** @copydoc KConfigSkeletonItem::readConfig */
-        void readConfig(KConfig *config) Q_DECL_OVERRIDE;
+        void readConfig(KConfig *config) override;
         /** @copydoc KConfigSkeletonItem::writeConfig */
-        void writeConfig(KConfig *config) Q_DECL_OVERRIDE;
+        void writeConfig(KConfig *config) override;
     };
 
     /**
@@ -959,19 +1088,19 @@ public:
                     const QList<QUrl> &defaultValue = QList<QUrl>());
 
         /** @copydoc KConfigSkeletonItem::readConfig(KConfig*) */
-        void readConfig(KConfig *config) Q_DECL_OVERRIDE;
+        void readConfig(KConfig *config) override;
 
         /** @copydoc KConfigSkeletonItem::writeConfig(KConfig*) */
-        void writeConfig(KConfig *config) Q_DECL_OVERRIDE;
+        void writeConfig(KConfig *config) override;
 
         /** @copydoc KConfigSkeletonItem::setProperty(const QVariant&) */
-        void setProperty(const QVariant &p) Q_DECL_OVERRIDE;
+        void setProperty(const QVariant &p) override;
 
         /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) const */
-        bool isEqual(const QVariant &p) const Q_DECL_OVERRIDE;
+        bool isEqual(const QVariant &p) const override;
 
         /** @copydoc KConfigSkeletonItem::property() */
-        QVariant property() const Q_DECL_OVERRIDE;
+        QVariant property() const override;
     };
 
     /**
@@ -986,16 +1115,16 @@ public:
                     const QList < int > &defaultValue = QList < int >());
 
         /** @copydoc KConfigSkeletonItem::readConfig(KConfig*) */
-        void readConfig(KConfig *config) Q_DECL_OVERRIDE;
+        void readConfig(KConfig *config) override;
 
         /** @copydoc KConfigSkeletonItem::setProperty(const QVariant&) */
-        void setProperty(const QVariant &p) Q_DECL_OVERRIDE;
+        void setProperty(const QVariant &p) override;
 
         /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) const */
-        bool isEqual(const QVariant &p) const Q_DECL_OVERRIDE;
+        bool isEqual(const QVariant &p) const override;
 
         /** @copydoc KConfigSkeletonItem::property() */
-        QVariant property() const Q_DECL_OVERRIDE;
+        QVariant property() const override;
     };
 
 public:
@@ -1040,12 +1169,13 @@ public:
      */
     void load();
 
-#ifndef KCONFIGCORE_NO_DEPRECATED
+#if KCONFIGCORE_ENABLE_DEPRECATED_SINCE(5, 0)
     /**
      * @deprecated since 5.0, call load() instead (to reload from disk) or just read()
      * if the underlying KConfig object is already up-to-date.
      */
-    KCONFIGCORE_DEPRECATED void readConfig()
+    KCONFIGCORE_DEPRECATED_VERSION(5, 0, "Use KCoreConfigSkeleton::load() or KCoreConfigSkeleton::read()")
+    void readConfig()
     {
         load();
     }
@@ -1063,6 +1193,21 @@ public:
      */
     void read();
 
+    /**
+     * Indicates if all the registered items are set to their default value.
+     *
+     * @since 5.64
+     */
+    bool isDefaults() const;
+
+    /**
+     * Indicates if any registered item has a different value than the
+     * previously loaded value.
+     *
+     * @since 5.64
+     */
+    bool isSaveNeeded() const;
+
     /**
      * Set the config file group for subsequent addItem() calls. It is valid
      * until setCurrentGroup() is called with a new argument. Call this before
@@ -1210,12 +1355,12 @@ public:
                                   qint64 defaultValue = 0,
                                   const QString &key = QString());
 
+#if KCONFIGCORE_ENABLE_DEPRECATED_SINCE(5, 0)
     /**
-     * @deprecated
-     * Use addItemLongLong().
+     * @deprecated Since 5.0, use addItemLongLong().
      */
-#ifndef KDE_NO_DEPRECATED
-    KCONFIGCORE_DEPRECATED ItemLongLong *addItemInt64(const QString &name, qint64 &reference,
+    KCONFIGCORE_DEPRECATED_VERSION(5, 0, "Use KCoreConfigSkeleton::addItemLongLong(...)")
+    ItemLongLong *addItemInt64(const QString &name, qint64 &reference,
             qint64 defaultValue = 0,
             const QString &key = QString());
 #endif
@@ -1235,12 +1380,12 @@ public:
                                     quint64 defaultValue = 0,
                                     const QString &key = QString());
 
+#if KCONFIGCORE_ENABLE_DEPRECATED_SINCE(5, 0)
     /**
-     * @deprecated
-     * Use addItemULongLong().
+     * @deprecated Since 5.0, use addItemULongLong().
      */
-#ifndef KDE_NO_DEPRECATED
-    KCONFIGCORE_DEPRECATED ItemULongLong *addItemUInt64(const QString &name, quint64 &reference,
+    KCONFIGCORE_DEPRECATED_VERSION(5, 0, "Use KCoreConfigSkeleton::addItemULongLong(...)")
+    ItemULongLong *addItemUInt64(const QString &name, quint64 &reference,
             quint64 defaultValue = 0,
             const QString &key = QString());
 #endif
@@ -1392,7 +1537,7 @@ public:
      * Return whether a certain item is immutable
      * @since 4.4
      */
-    bool isImmutable(const QString &name) const;
+    Q_INVOKABLE bool isImmutable(const QString &name) const;
 
     /**
      * Lookup item by name
@@ -1424,11 +1569,12 @@ public Q_SLOTS:
      */
     bool save();
 
-#ifndef KCONFIGCORE_NO_DEPRECATED
+#if KCONFIGCORE_ENABLE_DEPRECATED_SINCE(5, 0)
     /**
      * @deprecated since 5.0, call save() instead.
      */
-    KCONFIGCORE_DEPRECATED void writeConfig()
+    KCONFIGCORE_DEPRECATED_VERSION(5, 0, "Use KCoreConfigSkeleton::save()")
+    void writeConfig()
     {
         save();
     }
@@ -1468,21 +1614,27 @@ protected:
     /**
      * Perform the actual writing of the configuration file.
      * Override in derived classes to write special config values.
-     * Called from @ref writeConfig()
+     * Called from @ref save()
      */
     virtual bool usrSave();
 
+#if KCONFIGCORE_BUILD_DEPRECATED_SINCE(5, 0)
    /**
      * @deprecated since 5.0, override usrRead instead.  This method is still called from usrRead
      * for compatibility.
      */
-    KCONFIGCORE_DEPRECATED virtual void usrReadConfig();
+    KCONFIGCORE_DEPRECATED_VERSION(5, 0, "Override KCoreConfigSkeleton::usrRead()")
+    virtual void usrReadConfig();
+#endif
 
+#if KCONFIGCORE_BUILD_DEPRECATED_SINCE(5, 0)
    /**
      * @deprecated since 5.0, override usrSave instead.  This method is still called from usrSave
      * for compatibility.
      */
-    KCONFIGCORE_DEPRECATED virtual bool usrWriteConfig();
+    KCONFIGCORE_DEPRECATED_VERSION(5, 0, "Override KCoreConfigSkeleton::usrSave()")
+    virtual bool usrWriteConfig();
+#endif
 
 private:
     KCoreConfigSkeletonPrivate *const d;
